@@ -1,151 +1,144 @@
-'''Big Local News Python Client.'''
-from multiprocessing import Pool, cpu_count
-from http.client import responses
+"""Big Local News Python Client."""
 import json
+import logging
 import os
-import re
-import sys
 import platform
+import re
+from http.client import responses
+from multiprocessing import Pool, cpu_count
 
-import pandas as pd
 import requests
+
 from . import queries as q
 
+logger = logging.getLogger(__name__)
+
 
 class Client:
-    '''Big Local News Python Client.'''
-    def __init__(self, token, tier='prod', actor='user'):
-        '''Creates a Big Local News Python Client.
+    """Big Local News Python Client."""
+
+    def __init__(self, token=None, tier="prod"):
+        """Create a Big Local News Python Client.
 
         Args:
-            token: if the actor is 'user', this is a personal JWT; if it is
-                'user_plugin', this is a token received from the OAuth2 process;
-                lastly, if the actor is 'plugin', the token is a string in the
-                format '{client_id}:{secret}'
+            token: a personal token generated on the Big Local News website.
             tier: only 'prod' will work for external developers.
-            actor: ['user', 'user_plugin', 'plugin'] -- the authorization role
-                of the entity making graphql calls
 
         Returns:
             client: a Big Local News Python Client.
-        '''
+        """
+        if not token:
+            token = os.getenv("BLN_API_TOKEN")
+            if not token:
+                raise ValueError("No API token provided")
         self.token = token
         self.endpoint = {
-            'local': 'http://localhost:8080/graphql',
-            'dev': 'https://dev-api.biglocalnews.org/graphql',
-            'prod': 'https://api.biglocalnews.org/graphql',
+            "local": "http://localhost:8080/graphql",
+            "dev": "https://dev-api.biglocalnews.org/graphql",
+            "prod": "https://api.biglocalnews.org/graphql",
         }[tier]
-        self.auth_method = {
-            'user': 'JWT',
-            'user_plugin': 'Bearer',
-            'plugin': 'Basic',
-        }[actor]
 
-    def _gql(self, query, variables={}):
+    def _gql(self, query, variables=None):
+        variables = variables or {}
         # special case: node query, which doesn't use an *Input type
-        is_node = len(variables) == 1 and 'id' in variables
+        is_node = len(variables) == 1 and "id" in variables
         if not is_node:
             # other than node, only mutations use variables, and they all have
             # *Input object types, so nest variables inside 'input'; also,
             # remove 'self' so mutations can just pass 'locals()'
-            variables = {
-                'input': {k: v
-                          for k, v in variables.items() if k != 'self'}
-            }
-        data, err = _gql(self.endpoint, self.auth_method, self.token, query,
-                         variables)
+            variables = {"input": {k: v for k, v in variables.items() if k != "self"}}
+        data, err = _gql(self.endpoint, self.token, query, variables)
         # network error
         if err:
             return perr(err)
         if isinstance(data, dict):
-            for k, v in data.items():
+            for _k, v in data.items():
                 # unwrap single-item dict lists
                 if len(data) == 1 and isinstance(v, list):
                     return v
                 if isinstance(v, dict):
                     # mutation error
-                    if 'err' in v and v['err']:
-                        return perr(v['err'])
+                    if "err" in v and v["err"]:
+                        return perr(v["err"])
                     # mutation result
-                    if 'ok' in v:
-                        return v['ok']
+                    if "ok" in v:
+                        return v["ok"]
         # query result
         return data
 
-    def raw(self, query, variables={}, ungraphql=False):
-        '''Execute a raw query directly with variables.'''
-        data, err = _gql(self.endpoint, self.auth_method, self.token, query,
-                         variables, ungraphql)
+    def raw(self, query, variables=None, ungraphql=False):
+        """Execute a raw query directly with variables."""
+        data, err = _gql(self.endpoint, self.token, query, variables or {}, ungraphql)
         if err:
             return perr(err)
         return data
 
     def everything(self):
-        '''Returns all information accessible by the current user.'''
+        """Return all information accessible by the current user."""
         return self._gql(q.query_everything)
 
     def user(self):
-        '''Returns information about the current user.'''
+        """Return information about the current user."""
         return self._gql(q.query_user)
 
     def groupRoles(self):
-        '''Returns the current user's group roles and groups.'''
+        """Return the current user's group roles and groups."""
         return self._gql(q.query_groupRoles)
 
     def projectRoles(self):
-        '''Returns the current user's project roles and projects.'''
+        """Return the current user's project roles and projects."""
         return self._gql(q.query_projectRoles)
 
     def effectiveProjectRoles(self):
-        '''Returns the current user's effective project roles and projects.'''
+        """Return the current user's effective project roles and projects."""
         return self._gql(q.query_effectiveProjectRoles)
 
     def personalTokens(self):
-        '''Returns the current user's personal tokens.'''
+        """Return the current user's personal tokens."""
         return self._gql(q.query_personalTokens)
 
     def oauth2Codes(self):
-        '''Returns the current user's OAuth2 codes (authorized plugins).'''
+        """Return the current user's OAuth2 codes (authorized plugins)."""
         return self._gql(q.query_oauth2Codes)
 
     def oauth2Tokens(self):
-        '''Returns the current user's OAuth2 tokens (authorized plugins).'''
+        """Return the current user's OAuth2 tokens (authorized plugins)."""
         return self._gql(q.query_oauth2Tokens)
 
     def oauth2Clients(self):
-        '''Returns the current user's owned OAuth2 clients (plugins).'''
+        """Return the current user's owned OAuth2 clients (plugins)."""
         return self._gql(q.query_oauth2Clients)
 
     def userNames(self):
-        '''Returns a list of the user names on the platform.'''
+        """Return a list of the user names on the platform."""
         return self._gql(q.query_userNames)
 
     def groupNames(self):
-        '''Returns a list of the group names on the platform.'''
+        """Return a list of the group names on the platform."""
         return self._gql(q.query_groupNames)
 
     def openProjects(self):
-        '''Returns a list of open projects.'''
+        """Return a list of open projects."""
         return self._gql(q.query_openProjects)
 
     def oauth2ClientsPublic(self):
-        '''Returns a list of Public OAuth2 Clients, i.e. plugins.'''
+        """Return a list of Public OAuth2 Clients, i.e. plugins."""
         return self._gql(q.query_oauth2ClientsPublic)
 
     def authorizeOauth2Client(self, id, state):
-        '''Authorize an OAuth2 client by id with state.'''
+        """Authorize an OAuth2 client by id with state."""
         return self._gql(q.mutation_authorizeOauth2Client, locals())
 
     def authorizeWithPkceOauth2Client(self, id, state, codeChallenge):
-        '''Authorize an OAuth2 client by id with state and code challenge.'''
+        """Authorize an OAuth2 client by id with state and code challenge."""
         return self._gql(q.mutation_authorizeWithPkceOauth2Client, locals())
 
     def createFileDownloadUri(self, projectId, fileName):
-        '''Create a file download uri with a projectId and fileName.'''
+        """Create a file download uri with a projectId and fileName."""
         return self._gql(q.mutation_createFileDownloadUri, locals())
 
     def createFileUploadUri(self, projectId, fileName):
-        '''Create a file upload uri with a projectId and fileName.'''
+        """Create a file upload uri with a projectId and fileName."""
         return self._gql(q.mutation_createFileUploadUri, locals())
 
     def createGroup(
@@ -156,7 +149,7 @@ class Client:
         description=None,
         userRoles=None,
     ):
-        '''Creates a group.
+        """Create a group.
 
         Args:
             name: Group name.
@@ -169,7 +162,7 @@ class Client:
 
         Returns:
             group: the resulting group or None if error.
-        '''
+        """
         variables = {k: v for k, v in locals().items() if v}
         return self._gql(q.mutation_createGroup, variables)
 
@@ -183,7 +176,7 @@ class Client:
         scopes=None,
         pkceRequired=False,
     ):
-        '''Creates an OAuth2 Client (plugin).
+        """Create an OAuth2 Client (plugin).
 
         Args:
             name: Client name; must be unique.
@@ -200,12 +193,12 @@ class Client:
 
         Returns:
             client: the resulting client or None if error.
-        '''
+        """
         variables = {k: v for k, v in locals().items() if v}
         return self._gql(q.mutation_createOauth2Client, variables)
 
     def createPersonalToken(self):
-        '''Creates a personal token.'''
+        """Create a personal token."""
         return self._gql(q.mutation_createPersonalToken)
 
     def createProject(
@@ -218,9 +211,9 @@ class Client:
         userRoles=None,
         groupRoles=None,
         tags=None,
-        files=[],
+        files=None,
     ):
-        '''Creates a project.
+        """Create a project.
 
         Args:
             name: Project name.
@@ -238,71 +231,70 @@ class Client:
 
         Returns:
             project: the resulting project or None if error.
-        '''
-        variables = {k: v for k, v in locals().items() if k != 'files' and v}
+        """
+        variables = {k: v for k, v in locals().items() if k != "files" and v}
         project = self._gql(q.mutation_createProject, variables)
         if not project:
             return
-        self.upload_files(project['id'], files)
-        return self._gql(q.query_project, {'id': project['id']})
+        self.upload_files(project["id"], files or [])
+        return self._gql(q.query_project, {"id": project["id"]})
 
     def upload_files(self, projectId, files):
+        """Upload a files to the provided project id."""
         # run windows and mac uploads serially, for the following reasons:
         # windows will crash if MP is not guarded by main
         # mac has new fork rules: https://bugs.python.org/issue35219
-        if platform.system() != 'Linux':
+        if platform.system() != "Linux":
             for f in files:
-                _upload_file(self.endpoint, self.auth_method, self.token,
-                             projectId, f)
+                _upload_file(self.endpoint, self.token, projectId, f)
             return
         with Pool(cpu_count()) as p:
-            args = [(self.endpoint, self.auth_method, self.token, projectId, f)
-                    for f in files]
+            args = [(self.endpoint, self.token, projectId, f) for f in files]
             p.starmap(_upload_file, args)
 
     def upload_file(self, projectId, path):
-        '''Upload a file locally to a project.
+        """Upload a file locally to a project.
 
         Args:
             projectId: the id of the project.
             path: the path of the file to upload.
-        '''
+        """
         return self.upload_files(projectId, [path])
 
     def createTag(self, name):
-        '''Creates a tag.'''
+        """Create a tag."""
         self._gql(q.mutation_createTag, locals())
 
     def deleteFile(self, projectId, fileName):
-        '''Deletes `filename` from `projectId`.'''
+        """Delete `filename` from `projectId`."""
         return self._gql(q.mutation_deleteFile, locals())
 
     def deleteProject(self, id):
-        '''Deletes project `projectId`.'''
+        """Delete project `projectId`."""
         return self._gql(q.mutation_deleteProject, locals())
 
     def deleteOauth2Client(self, id):
-        '''Deletes OAuth2 Client with id `id`.'''
+        """Delete OAuth2 Client with id `id`."""
         return self._gql(q.mutation_deleteOauth2Client, locals())
 
     def exchangeOauth2CodeForToken(self, code):
-        '''Exchanges an OAuth2 code for a token.'''
+        """Exchange an OAuth2 code for a token."""
         return self._gql(q.mutation_exchangeOauth2CodeForToken, locals())
 
     def exchangeOauth2CodeWithPkceForToken(self, code, codeVerifier):
-        '''Exchanges an OAuth2 code and code verifier for a token.'''
+        """Exchange an OAuth2 code and code verifier for a token."""
         return self._gql(q.mutation_exchangeOauth2CodeForToken, locals())
 
     def revokeOauth2Token(self, token):
-        '''Revokes an OAuth2 token (used by clients).'''
+        """Revoke an OAuth2 token (used by clients)."""
         return self._gql(q.mutation_revokeOauth2Token, locals())
 
     def revokePersonalTokens(self, token):
-        '''Revokes a Personal Tokens.'''
+        """Revoke a Personal Tokens."""
         return self._gql(q.mutation_revokePersonalToken, locals())
 
     def unauthorizeOauth2Client(self, id):
-        '''Unauthorizes an OAuth2 client by id.'''
+        """Unauthorize an OAuth2 client by id."""
         return self._gql(q.mutation_unauthorizeOauth2Client, locals())
 
     def updateGroup(
@@ -314,7 +306,7 @@ class Client:
         description=None,
         userRoles=None,
     ):
-        '''Updates a group.
+        """Update a group.
 
         Args:
             id: ID of group to update.
@@ -326,7 +318,7 @@ class Client:
 
         Returns:
             group: the resulting group or None if error.
-        '''
+        """
         variables = {k: v for k, v in locals().items() if v}
         return self._gql(q.mutation_updateGroup, variables)
 
@@ -341,7 +333,7 @@ class Client:
         scopes=None,
         pkceRequired=False,
     ):
-        '''Update an OAuth2 Client (plugin).'''
+        """Update an OAuth2 Client (plugin)."""
         variables = {k: v for k, v in locals().items() if v}
         return self._gql(q.mutation_updateOauth2Client, variables)
 
@@ -356,9 +348,9 @@ class Client:
         userRoles=None,
         groupRoles=None,
         tags=None,
-        files=[],
+        files=None,
     ):
-        '''Updates a project.
+        """Update a project.
 
         Args:
             id: ID of project to update.
@@ -377,9 +369,9 @@ class Client:
 
         Returns:
             project: the resulting project or None if error.
-        '''
-        variables = {k: v for k, v in locals().items() if k != 'files' and v}
-        self.upload_files(id, files)
+        """
+        variables = {k: v for k, v in locals().items() if k != "files" and v}
+        self.upload_files(id, files or [])
         return self._gql(q.mutation_updateProject, variables)
 
     def updateUser(
@@ -390,7 +382,7 @@ class Client:
         contact=None,
         contactMethod=None,
     ):
-        '''Updates a user.
+        """Update a user.
 
         Args:
             id: id of user to update.
@@ -401,14 +393,58 @@ class Client:
 
         Returns:
             user: the resulting user or None if error.
-        '''
+        """
         variables = {k: v for k, v in locals().items() if v}
         return self._gql(q.mutation_updateUser, variables)
 
     # python SDK convenience functions
 
+    def get_project_by_id(self, id: str):
+        """Get the project with the provided id.
+
+        Args:
+            name (str): The name of the project on biglocalnews.org
+
+        Returns: Dictionary with project metadata.
+        """
+        # Search all projects by name
+        project_list = self.search_projects(lambda x: x["id"] == id)
+
+        # If there's no results, throw an error
+        if len(project_list) == 0:
+            raise ValueError(f"No project with {id} id found")
+
+        # If there's more than one project, throw an error
+        if len(project_list) > 1:
+            raise ValueError(f"{len(project_list)} projects with {id} id found")
+
+        # Otherwise, return the one project found
+        return project_list[0]
+
+    def get_project_by_name(self, name: str):
+        """Get the project with the provided name.
+
+        Args:
+            name (str): The name of the project on biglocalnews.org
+
+        Returns: Dictionary with project metadata.
+        """
+        # Search all projects by name
+        project_list = self.search_projects(lambda x: x["name"] == name)
+
+        # If there's no results, throw an error
+        if len(project_list) == 0:
+            raise ValueError(f"No project named {name} found")
+
+        # If there's more than one project, throw an error
+        if len(project_list) > 1:
+            raise ValueError(f"{len(project_list)} projects named {name} found")
+
+        # Otherwise, return the one project found
+        return project_list[0]
+
     def download_file(self, projectId, filename, output_dir=None):
-        '''Downloads `filename` in project `projectId` to `output_dir`.
+        """Download `filename` in project `projectId` to `output_dir`.
 
         Args:
             projectId: the id of a Big Local News project.
@@ -417,86 +453,48 @@ class Client:
 
         Returns:
             ouput_path: location where file was saved or None if error.
-        '''
-        output_dir = os.path.expanduser(output_dir)
+        """
         if not output_dir:
             output_dir = os.getcwd()
+        output_dir = os.path.expanduser(output_dir)
         uri = self.createFileDownloadUri(projectId, filename)
         if not uri:
             return
-        with requests.get(uri['uri'], stream=True) as r:
+        with requests.get(uri["uri"], stream=True) as r:
             if r.status_code != requests.codes.ok:
                 return perr(responses[r.status_code])
             output_path = os.path.join(output_dir, filename)
-            with open(output_path, 'wb') as f:
+            with open(output_path, "wb") as f:
                 for chunk in r.iter_content(chunk_size=8192):
                     if chunk:  # filter out keep-alive new chunks
                         f.write(chunk)
             return output_path
 
     def upload_from_json(self, json_path):
-        '''Uploads groups and projects from a json config.
+        """Upload groups and projects from a json config.
 
         See example_config.json.
-        '''
+        """
         path = os.path.expanduser(json_path)
         if not os.path.exists(path):
-            return perr(f'invalid json_path: {path}')
+            return perr(f"invalid json_path: {path}")
         with open(path) as f:
             data = json.load(f)
-        if 'groups' in data:
-            for group in data['groups']:
-                if 'id' in group:
+        if "groups" in data:
+            for group in data["groups"]:
+                if "id" in group:
                     self.createGroup(**group)
                 else:
                     self.updateGroup(**group)
-        if 'projects' in data:
-            for project in data['projects']:
-                if 'id' in project:
+        if "projects" in data:
+            for project in data["projects"]:
+                if "id" in project:
                     self.updateProject(**project)
                 else:
                     self.createProject(**project)
 
-    def file_to_pandas(self, projectId, fileName):
-        '''Returns a pandas DataFrame of `fileName` in project `projectId`.
-
-        Args:
-            projectId: the id of the project.
-            fileName: the remote file name.
-
-        Returns:
-            df: a pandas DataFrame.
-        '''
-        uri = self.createFileDownloadUri(projectId, fileName)
-        if not uri:
-            return
-        return self._uri_to_pandas(uri)
-
-    def _uri_to_pands(self, uri):
-        if _is_excel(uri['name']):
-            return pd.read_excel(uri['uri'])
-        # sep=None and engine='python' tells pandas to detect the type
-        return pd.read_table(uri['uri'], sep=None, engine='python')
-
-    def pandas_to_csv(self, df, projectId, fileName):
-        '''Uploads a pandas DataFrame to project `projectId` as `fileName`.
-
-        Args:
-            df: a pandas DataFrame.
-            projectId: the id of the project.
-            fileName: the remote filename.
-        '''
-        if not isinstance(df, pd.DataFrame):
-            return perr('Can only upload pandas DataFrames.')
-        if os.path.splitext(fileName)[1] != '.csv':
-            return perr('Must upload to a file with a csv extension.')
-        uri = self.createFileUploadUri(projectId, fileName)
-        if not uri:
-            return
-        return _put_string(df.to_csv(index=False), uri['uri'])
-
-    def search_groups(self, predicate=lambda g: re.match('.*', g['name'])):
-        '''Returns groups where `predicate(group)` is True.
+    def search_groups(self, predicate=lambda g: re.match(".*", g["name"])):
+        """Return groups where `predicate(group)` is True.
 
         Args:
             predicate: (optional) a function that takes a group and returns
@@ -505,15 +503,15 @@ class Client:
 
         Returns:
             groups: list of groups where `predicate(group)` is true.
-        '''
+        """
         groups = []
         for v in self.groupRoles():
-            if predicate(v['group']):
-                groups.append(v['group'])
+            if predicate(v["group"]):
+                groups.append(v["group"])
         return groups
 
-    def search_projects(self, predicate=lambda p: re.match('.*', p['name'])):
-        '''Returns projects where `predicate(project)` is True.
+    def search_projects(self, predicate=lambda p: re.match(".*", p["name"])):
+        """Return projects where `predicate(project)` is True.
 
         Args:
             predicate: (optional) a function that takes a project and returns
@@ -522,15 +520,15 @@ class Client:
 
         Returns:
             projects: list of projects where `predicate(project)` is true.
-        '''
+        """
         projects = []
         for v in self.effectiveProjectRoles():
-            if predicate(v['project']):
-                projects.append(v['project'])
+            if predicate(v["project"]):
+                projects.append(v["project"])
         return projects
 
-    def search_files(self, predicate=lambda f: re.match('.*', f['name'])):
-        '''Returns projects where `predicate(file)` is True.
+    def search_files(self, predicate=lambda f: re.match(".*", f["name"])):
+        """Return projects where `predicate(file)` is True.
 
         Args:
             predicate: (optional) a function that takes a file object and
@@ -541,61 +539,26 @@ class Client:
 
         Returns:
             files: list of file objects where `predicate(file_obj)` is true.
-        '''
+        """
         files = []
         for v in self.effectiveProjectRoles():
-            for f in v['project']['files']:
+            for f in v["project"]["files"]:
                 if predicate(f):
-                    f['projectId'] = v['project']['id']
-                    f['projectName'] = v['project']['name']
+                    f["projectId"] = v["project"]["id"]
+                    f["projectName"] = v["project"]["name"]
                     files.append(f)
         return files
 
-    def search_to_pandas(
-            self,
-            file_predicate=lambda f: re.match('.*', f['name']),
-            project_predicate=lambda p: re.match('.*', p['name']),
-    ):
-        '''Returns matching project/file name regex to pandas DataFrame.
-
-        Args:
-            file_predicate: (optional) a function that takes a File object and
-                returns True or False.
-            project_predicate: (optional) a function that takes a project and
-                returns True or False.
-
-            The for those files that match both the `file_predicate` and
-            the `project_predicate`, the user is asked to select one to load
-            into pandas.
-
-        Returns:
-            df: a pandas DataFrame.
-        '''
-        uris = []
-        for v in self.effectiveProjectRoles():
-            if project_predicate(v['project']):
-                for f in v['project']['files']:
-                    if file_predicate(f):
-                        f['projectId'] = v['project']['id']
-                        f['projectName'] = v['project']['name']
-                        uris.append(f)
-        if not uris:
-            return perr('No matching files')
-        idx = _select_idx([uri['name'] for uri in uris])
-        uri = dict(enumerate(uris))[idx]
-        return self._uri_to_pands(uri)
-
 
 def _gql(
     endpoint,
-    auth_method,
     token,
     query_string,
-    variables={},
+    variables=None,
     ungraphql=True,
 ):
-    inpt = {'query': query_string, 'variables': variables}
-    headers = {'Authorization': f'{auth_method} {token}'}
+    inpt = {"query": query_string, "variables": variables or {}}
+    headers = {"Authorization": f"JWT {token}"}
     res = requests.post(endpoint, json=inpt, headers=headers)
     if res.status_code != requests.codes.ok:
         return None, responses[res.status_code]
@@ -607,18 +570,18 @@ def _gql(
 
 def _ungraphql(root):
     # unwraps levels that are simply (id, <item>: <value>) to just <value>
-    if isinstance(root, dict) and 'id' in root and len(root) == 2:
+    if isinstance(root, dict) and "id" in root and len(root) == 2:
         for k, v in root.items():
-            if k != 'id':
+            if k != "id":
                 return _ungraphql(v)
-    if isinstance(root, dict) and 'data' in root:
-        return _ungraphql(root['data'])
-    if isinstance(root, dict) and 'user' in root:
-        return _ungraphql(root['user'])
-    if isinstance(root, dict) and 'node' in root:
-        return _ungraphql(root['node'])
-    if isinstance(root, dict) and 'edges' in root:
-        return _ungraphql(root['edges'])
+    if isinstance(root, dict) and "data" in root:
+        return _ungraphql(root["data"])
+    if isinstance(root, dict) and "user" in root:
+        return _ungraphql(root["user"])
+    if isinstance(root, dict) and "node" in root:
+        return _ungraphql(root["node"])
+    if isinstance(root, dict) and "edges" in root:
+        return _ungraphql(root["edges"])
     if isinstance(root, dict):
         d = {}
         for k, v in root.items():
@@ -629,41 +592,46 @@ def _ungraphql(root):
     return root
 
 
-def _upload_file(endpoint, auth_method, token, projectId, path):
-    print(f'uploading {path}')
+def _upload_file(endpoint, token, projectId, path):
+    logger.debug(f"uploading {path}")
     path = os.path.expanduser(path)
     if not os.path.exists(path):
-        return perr(f'invalid path: {path}')
-    uri, err = _get_upload_uri(endpoint, auth_method, token, projectId, path)
+        return perr(f"invalid path: {path}")
+    uri, err = _get_upload_uri(endpoint, token, projectId, path)
     if err:
         return perr(err)
-    err = _put(path, uri['uri'])
+    err = _put(path, uri["uri"])
     if err:
         return perr(err)
 
 
-def _get_upload_uri(endpoint, auth_method, token, projectId, path):
+def _get_upload_uri(endpoint, token, projectId, path):
     fname = os.path.basename(path)
-    data, err = _gql(endpoint, auth_method, token,
-                     q.mutation_createFileUploadUri,
-                     {'input': {
-                         'projectId': projectId,
-                         'fileName': fname,
-                     }})
+    data, err = _gql(
+        endpoint,
+        token,
+        q.mutation_createFileUploadUri,
+        {
+            "input": {
+                "projectId": projectId,
+                "fileName": fname,
+            }
+        },
+    )
     if err:
         return None, err
-    data = data['createFileUploadUri']
-    if data['err']:
-        return None, data['err']
-    return data['ok'], None
+    data = data["createFileUploadUri"]
+    if data["err"]:
+        return None, data["err"]
+    return data["ok"], None
 
 
 def _put(path, uri):
     headers = {
-        'content-type': 'application/octet-stream',
-        'host': 'storage.googleapis.com',
+        "content-type": "application/octet-stream",
+        "host": "storage.googleapis.com",
     }
-    with open(path, 'rb') as f:
+    with open(path, "rb") as f:
         res = requests.put(uri, data=f, headers=headers)
         if res.status_code != requests.codes.ok:
             return responses[res.status_code]
@@ -671,22 +639,18 @@ def _put(path, uri):
 
 def _put_string(string, uri):
     headers = {
-        'content-type': 'application/octet-stream',
-        'host': 'storage.googleapis.com',
+        "content-type": "application/octet-stream",
+        "host": "storage.googleapis.com",
     }
-    res = requests.put(uri, data=string.encode('utf-8'), headers=headers)
+    res = requests.put(uri, data=string.encode("utf-8"), headers=headers)
     if res.status_code != requests.codes.ok:
         return responses[res.status_code]
 
 
-def _is_excel(filename):
-    return os.path.splitext(filename)[1] in ['.xls', '.xlsx']
-
-
 def _select_idx(options):
     for idx, option in enumerate(options):
-        print(f'{idx}: {option}')
-    msg = 'Select index: '
+        logger.debug(f"{idx}: {option}")
+    msg = "Select index: "
     idx = _to_idx(input(msg))
     while idx < 0 or idx >= len(options):
         idx = _to_idx(input(msg))
@@ -700,5 +664,6 @@ def _to_idx(s):
         return -1
 
 
-def perr(msg, end='\n'):
-    print(msg, file=sys.stderr, end=end)
+def perr(msg, end="\n"):
+    """Print error to stdout."""
+    logger.error(msg)
